"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.flushPassiveEffects = flushPassiveEffects;
exports.default = void 0;

var _host = _interopRequireDefault(require("./host"));

var _scheduler = require("./scheduler");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function enqueueCallback(internal, callback) {
  if (callback) {
    var callbackQueue = internal._pendingCallbacks || (internal._pendingCallbacks = []);
    callbackQueue.push(callback);
  }
}

function enqueueState(internal, partialState) {
  if (partialState) {
    var stateQueue = internal._pendingStateQueue || (internal._pendingStateQueue = []);
    stateQueue.push(partialState);
  }
}

function runCallbacks(callbacks, context) {
  if (callbacks) {
    for (var i = 0; i < callbacks.length; i++) {
      callbacks[i].call(context);
    }
  }
}

function runUpdate(component) {
  var internal = component._internal;

  if (!internal) {
    return;
  }

  _host.default.isRendering = true; // If updateComponent happens to enqueue any new updates, we
  // shouldn't execute the callbacks until the next render happens, so
  // stash the callbacks first

  var callbacks = internal._pendingCallbacks;
  internal._pendingCallbacks = null;
  var prevElement = internal._currentElement;
  var prevUnmaskedContext = internal._context;
  var nextUnmaskedContext = internal._penddingContext || prevUnmaskedContext;
  internal._penddingContext = undefined;

  if (internal._pendingStateQueue || internal._pendingForceUpdate) {
    internal.updateComponent(prevElement, prevElement, prevUnmaskedContext, nextUnmaskedContext);
  }

  runCallbacks(callbacks, component);
  _host.default.isRendering = false;
}

function mountOrderComparator(c1, c2) {
  return c2._internal._mountID - c1._internal._mountID;
}

function performWork() {
  if (_host.default.isRendering) {
    return;
  }

  var dirtyComponents;
  var component;

  while (_host.default.dirtyComponents.length > 0) {
    // Before next render, we will flush all the PassiveEffects
    flushPassiveEffects(); // Stash the dirtyComponents first

    dirtyComponents = _host.default.dirtyComponents; // Since reconciling a component higher in the owner hierarchy usually (not
    // always -- see shouldComponentUpdate()) will reconcile children, reconcile
    // them before their children by sorting the array.

    dirtyComponents.sort(mountOrderComparator); // Reset Host.dirtyComponents, using to collect dirtyComponent
    // generated by the next render

    _host.default.dirtyComponents = [];

    while (component = dirtyComponents.pop()) {
      runUpdate(component);
    }
  }
}

function scheduleWork(component) {
  var dirtyComponents = _host.default.dirtyComponents;

  if (dirtyComponents.indexOf(component) < 0) {
    dirtyComponents.push(component);
  }

  performWork();
}

function flushPassiveEffects() {
  (0, _scheduler.flushBeforeNextRenderCallbacks)();
}

function updateState(component, partialState, callback) {
  var internal = component._internal;

  if (!internal) {
    return;
  }

  !_host.default.isRendering && flushPassiveEffects();
  enqueueCallback(internal, callback);
  var hasComponentRendered = internal._renderedComponent; // setState

  if (partialState) {
    enqueueState(internal, partialState); // State pending in componentWillReceiveProps and componentWillMount

    if (!internal._pendingState && hasComponentRendered) {
      scheduleWork(component);
    }
  } else {
    // forceUpdate
    internal._pendingForceUpdate = true;

    if (hasComponentRendered) {
      scheduleWork(component);
    }
  }
}

(0, _scheduler.setPerformWork)(performWork);
var Updater = {
  setState: updateState,
  forceUpdate: function forceUpdate(component, callback) {
    updateState(component, null, callback);
  },
  runCallbacks: runCallbacks
};
var _default = Updater;
exports.default = _default;